"""
File: /__init__.py
Created Date: Saturday August 23rd 2025
Author: Christian Nonis <alch.infoemail@gmail.com>
-----
Last Modified: Saturday August 23rd 2025 2:45:17 pm
Modified By: the developer formerly known as Christian Nonis at <alch.infoemail@gmail.com>
-----
"""
